var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var OS;
(function (OS) {
    // import the CodePad application module
    const App = OS.application.CodePad;
    /**
     *
     *
     * @class AntOSDK
     * @extends {App.BaseExtension}
     */
    class AntOSDK extends App.BaseExtension {
        /**
         *Creates an instance of AntOSDK.
         * @param {application.CodePad} app
         * @memberof AntOSDK
         */
        constructor(app) {
            super(app);
        }
        // public functions
        /**
         *
         *
         * @returns
         * @memberof AntOSDK
         */
        create() {
            this.app
                .openDialog("FileDialog", {
                title: "__(New Project at)",
                file: { basename: __("ProjectName") },
                mimes: ["dir"],
            })
                .then((d) => {
                this.logger().clear();
                return this.mktpl(d.file.path, d.name, true);
            });
        }
        /**
         *
         *
         * @returns {void}
         * @memberof AntOSDK
         */
        init() {
            this.logger().clear();
            const dir = this.app.currdir;
            if (!dir || !dir.basename) {
                return this.create();
            }
            dir.read().then((d) => {
                if (d.error) {
                    return this.logger().error(__("Cannot read folder: {0}", dir.path));
                }
                if (d.result.length !== 0) {
                    return this.logger().error(__("The folder is not empty: {0}", dir.path));
                }
                this.mktpl(dir.parent().path, dir.basename);
            });
        }
        /**
         *
         *
         * @memberof AntOSDK
         */
        buildnrun() {
            this.logger().clear();
            this.metadata("project.json")
                .then((meta) => __awaiter(this, void 0, void 0, function* () {
                try {
                    yield this.build(meta, true);
                    try {
                        return this.run(meta);
                    }
                    catch (e) {
                        return this.logger().error(__("Unable to run project: {0}", e.stack));
                    }
                }
                catch (e_1) {
                    return this.logger().error(__("Unable to build project: {0}", e_1.stack));
                }
            }))
                .catch((e) => this.logger().error(__("Unable to read meta-data: {0}", e.stack)));
        }
        /**
         *
         *
         * @memberof AntOSDK
         */
        release() {
            this.logger().clear();
            this.metadata("project.json")
                .then((meta) => __awaiter(this, void 0, void 0, function* () {
                try {
                    yield this.build(meta, false);
                    try {
                        return this.mkar(`${meta.root}/build/debug`, `${meta.root}/build/release/${meta.name}.zip`);
                    }
                    catch (e) {
                        return this.logger().error(__("Unable to create package archive: {0}", e.stack));
                    }
                }
                catch (e_1) {
                    return this.logger().error(__("Unable to build project: {0}", e_1.stack));
                }
            }))
                .catch((e) => this.logger().error(__("Unable to read meta-data: {0}", e.stack)));
        }
        // private functions
        /**
         *
         *
         * @private
         * @param {string} path
         * @param {string} name
         * @param {boolean} [flag]
         * @memberof AntOSDK
         */
        mktpl(path, name, flag) {
            const rpath = `${path}/${name}`;
            const dirs = [
                `${rpath}/javascripts`,
                `${rpath}/css`,
                `${rpath}/coffees`,
                `${rpath}/assets`,
            ];
            if (flag) {
                dirs.unshift(rpath);
            }
            const files = [
                ["templates/sdk-main.tpl", `${rpath}/coffees/main.coffee`],
                ["templates/sdk-package.tpl", `${rpath}/package.json`],
                ["templates/sdk-project.tpl", `${rpath}/project.json`],
                ["templates/sdk-README.tpl", `${rpath}/README.md`],
                ["templates/sdk-scheme.tpl", `${rpath}/assets/scheme.html`],
            ];
            this.mkdirAll(dirs)
                .then(() => __awaiter(this, void 0, void 0, function* () {
                try {
                    yield this.mkfileAll(files, path, name);
                    this.app.currdir = rpath.asFileHandle();
                    this.app.toggleSideBar();
                    return this.app.eum.active.openFile(`${rpath}/README.md`.asFileHandle());
                }
                catch (e) {
                    return this.logger().error(__("Unable to create template files: {0}", e.stack));
                }
            }))
                .catch((e) => this.logger().error(__("Unable to create project directory: {0}", e.stack)));
        }
        /**
         *
         *
         * @private
         * @param {string[]} list
         * @returns {Promise<void>}
         * @memberof AntOSDK
         */
        verify(list) {
            return new Promise((resolve, reject) => {
                if (list.length === 0) {
                    return resolve();
                }
                const file = list.splice(0, 1)[0].asFileHandle();
                this.logger().info(__("Verifying: {0}", file.path));
                return file
                    .read()
                    .then((data) => {
                    try {
                        CoffeeScript.nodes(data);
                        return this.verify(list)
                            .then(() => resolve())
                            .catch((e) => reject(__e(e)));
                    }
                    catch (ex) {
                        return reject(__e(ex));
                    }
                })
                    .catch((e) => reject(__e(e)));
            });
        }
        /**
         *
         *
         * @private
         * @param {GenericObject<any>} meta
         * @returns {Promise<any>}
         * @memberof AntOSDK
         */
        compile(meta) {
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                try {
                    yield this.import([
                        `${this.basedir()}/libs/coffeescript.js`,
                        `${this.basedir()}/libs/terser.min.js`,
                    ]);
                    const list = meta.coffees.map((v) => `${meta.root}/${v}`);
                    try {
                        yield this.verify(list.map((x) => x));
                        try {
                            const code = yield this.cat(list, "");
                            const jsrc = CoffeeScript.compile(code);
                            this.logger().info(__("Compiled successful"));
                            return resolve(jsrc);
                        }
                        catch (e) {
                            return reject(__e(e));
                        }
                    }
                    catch (e_1) {
                        return reject(__e(e_1));
                    }
                }
                catch (e_2) {
                    return reject(__e(e_2));
                }
            }));
        }
        /**
         *
         *
         * @private
         * @param {GenericObject<any>} meta
         * @param {boolean} debug
         * @returns {Promise<void>}
         * @memberof AntOSDK
         */
        build(meta, debug) {
            const dirs = [
                `${meta.root}/build`,
                `${meta.root}/build/debug`,
                `${meta.root}/build/release`,
            ];
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                try {
                    yield this.mkdirAll(dirs);
                    try {
                        const src = yield this.compile(meta);
                        let v;
                        try {
                            let jsrc = yield this.cat((() => {
                                const result = [];
                                for (v of meta.javascripts) {
                                    result.push(`${meta.root}/${v}`);
                                }
                                return result;
                            })(), src);
                            yield new Promise(function (r, e) {
                                return __awaiter(this, void 0, void 0, function* () {
                                    let code = jsrc;
                                    if (!debug) {
                                        const options = {
                                            toplevel: true,
                                            compress: {
                                                passes: 3,
                                            },
                                            mangle: true,
                                            output: {
                                            //beautify: true,
                                            },
                                        };
                                        const result_1 = Terser.minify(jsrc, options);
                                        if (result_1.error) {
                                            this.logger().error(__("Unable to minify code: {0}", result_1.error));
                                        }
                                        else {
                                            ({ code } = result_1);
                                        }
                                    }
                                    try {
                                        const d = yield `${meta.root}/build/debug/main.js`
                                            .asFileHandle()
                                            .setCache(code)
                                            .write("text/plain");
                                        return r();
                                    }
                                    catch (ex) {
                                        return e(__e(ex));
                                    }
                                });
                            });
                            yield new Promise((r, e) => __awaiter(this, void 0, void 0, function* () {
                                const txt = yield this.cat((() => {
                                    const result1 = [];
                                    for (v of meta.css) {
                                        result1.push(`${meta.root}/${v}`);
                                    }
                                    return result1;
                                })(), "");
                                if (txt === "") {
                                    return r();
                                }
                                try {
                                    const d_1 = yield `${meta.root}/build/debug/main.css`
                                        .asFileHandle()
                                        .setCache(txt)
                                        .write("text/plain");
                                    return r();
                                }
                                catch (ex_1) {
                                    return e(__e(ex_1));
                                }
                            }));
                            yield this.copy((() => {
                                const result1_1 = [];
                                for (v of meta.copies) {
                                    result1_1.push(`${meta.root}/${v}`);
                                }
                                return result1_1;
                            })(), `${meta.root}/build/debug`);
                            return resolve();
                        }
                        catch (e) {
                            return reject(__e(e));
                        }
                    }
                    catch (e_1) {
                        return reject(__e(e_1));
                    }
                }
                catch (e_2) {
                    return reject(__e(e_2));
                }
            }));
        }
        /**
         *
         *
         * @private
         * @param {GenericObject<any>} meta
         * @memberof AntOSDK
         */
        run(meta) {
            `${meta.root}/build/debug/package.json`
                .asFileHandle()
                .read("json")
                .then((v) => {
                v.text = v.name;
                v.path = `${meta.root}/build/debug`;
                v.filename = meta.name;
                v.type = "app";
                v.mime = "antos/app";
                if (v.icon) {
                    v.icon = `${v.path}/${v.icon}`;
                }
                if (!v.iconclass && !v.icon) {
                    v.iconclass = "fa fa-adn";
                }
                this.logger().info(__("Installing..."));
                OS.setting.system.packages[meta.name] = v;
                this.logger().info(__("Running {0}...", meta.name));
                return OS.GUI.forceLaunch(meta.name, []);
            });
        }
    }
    App.extensions.AntOSDK = AntOSDK;
})(OS || (OS = {}));
