var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var OS;
(function (OS) {
    // import the CodePad application module
    const App = OS.application.CodePad;
    /**
     *
     *
     * @class ExtensionMaker
     * @extends {App.BaseExtension}
     */
    class ExtensionMaker extends App.BaseExtension {
        constructor(app) {
            super(app);
        }
        // public functions
        /**
         *
         *
         * @memberof ExtensionMaker
         */
        create() {
            this.app
                .openDialog("FileDialog", {
                title: "__(New CodePad extension at)",
                file: { basename: __("ExtensionName") },
                mimes: ["dir"],
            })
                .then((d) => {
                return this.mktpl(d.file.path, d.name);
            });
        }
        /**
         *
         *
         * @memberof ExtensionMaker
         */
        buildnrun() {
            this.metadata("extension.json")
                .then((meta) => __awaiter(this, void 0, void 0, function* () {
                try {
                    yield this.build(meta);
                    try {
                        return this.run(meta);
                    }
                    catch (e) {
                        return this.error(__("Unable to run extension"), e);
                    }
                }
                catch (e_1) {
                    return this.error(__("Unable to build extension"), e_1);
                }
            }))
                .catch((e) => this.error(__("Unable to read meta-data"), e));
        }
        /**
         *
         *
         * @memberof ExtensionMaker
         */
        release() {
            this.metadata("extension.json")
                .then((meta) => __awaiter(this, void 0, void 0, function* () {
                try {
                    yield this.build(meta);
                    try {
                        return this.mkar(`${meta.root}/build/debug`, `${meta.root}/build/release/${meta.meta.name}.zip`);
                    }
                    catch (e) {
                        return this.error(__("Unable to create archive"), e);
                    }
                }
                catch (e_1) {
                    return this.error(__("Unable to build extension"), e_1);
                }
            }))
                .catch((e) => this.error(__("Unable to read meta-data"), e));
        }
        /**
         *
         *
         * @memberof ExtensionMaker
         */
        install() {
            this.app
                .openDialog("FileDialog", {
                title: "__(Select extension archive)",
                mimes: [".*/zip"],
            })
                .then((d) => __awaiter(this, void 0, void 0, function* () {
                try {
                    yield this.installZip(d.file.path);
                    this.notify(__("Extension installed"));
                    return this.app.loadExtensionMetaData();
                }
                catch (e) {
                    return this.error(__("Unable to install extension"), e);
                }
            }));
        }
        /**
         *
         *
         * @private
         * @param {string} path
         * @param {string} name
         * @memberof ExtensionMaker
         */
        mktpl(path, name) {
            const rpath = `${path}/${name}`;
            const dirs = [
                rpath,
                `${rpath}/build`,
                `${rpath}/build/release`,
                `${rpath}/build/debug`,
            ];
            const files = [
                ["templates/ext-main.tpl", `${rpath}/${name}.coffee`],
                ["templates/ext-extension.tpl", `${rpath}/extension.json`],
            ];
            this.mkdirAll(dirs)
                .then(() => __awaiter(this, void 0, void 0, function* () {
                try {
                    yield this.mkfileAll(files, path, name);
                    this.app.currdir = rpath.asFileHandle();
                    this.app.toggleSideBar();
                    return this.app.openFile(`${rpath}/${name}.coffee`.asFileHandle());
                }
                catch (e) {
                    return this.error(__("Unable to create extension template"), e);
                }
            }))
                .catch((e) => this.error(__("Unable to create extension directories"), e));
        }
        /**
         *
         *
         * @private
         * @param {string[]} list
         * @returns {Promise<any>}
         * @memberof ExtensionMaker
         */
        verify(list) {
            return new Promise((resolve, reject) => {
                if (list.length === 0) {
                    return resolve();
                }
                const file = list.splice(0, 1)[0].asFileHandle();
                this.notify(__("Verifying: {0}", file.path));
                return file
                    .read()
                    .then((data) => {
                    try {
                        CoffeeScript.nodes(data);
                        return this.verify(list)
                            .then(() => resolve())
                            .catch((e) => reject(__e(e)));
                    }
                    catch (ex) {
                        return reject(__e(ex));
                    }
                })
                    .catch((e) => reject(__e(e)));
            });
        }
        /**
         *
         *
         * @private
         * @param {GenericObject<any>} meta
         * @returns {Promise<any>}
         * @memberof ExtensionMaker
         */
        compile(meta) {
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                try {
                    yield this.import([`${this.basedir()}/libs/coffeescript.js`]);
                    const list = meta.coffees.map((v) => `${meta.root}/${v}`);
                    try {
                        yield this.verify(list.map((x) => x));
                        try {
                            const code = yield this.cat(list, "");
                            const jsrc = CoffeeScript.compile(code);
                            this.notify(__("Compiled successful"));
                            return resolve(jsrc);
                        }
                        catch (e) {
                            return reject(__e(e));
                        }
                    }
                    catch (e_1) {
                        return reject(__e(e_1));
                    }
                }
                catch (e_2) {
                    return reject(__e(e_2));
                }
            }));
        }
        /**
         *
         *
         * @private
         * @param {GenericObject<any>} meta
         * @returns {Promise<any>}
         * @memberof ExtensionMaker
         */
        build(meta) {
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                try {
                    const src = yield this.compile(meta);
                    let v;
                    try {
                        const jsrc = yield this.cat((() => {
                            const result = [];
                            for (v of meta.javascripts) {
                                result.push(`${meta.root}/${v}`);
                            }
                            return result;
                        })(), src);
                        yield new Promise((r, e) => `${meta.root}/build/debug/${meta.meta.name}.js`
                            .asFileHandle()
                            .setCache(jsrc)
                            .write("text/plain")
                            .then((d) => r())
                            .catch((ex) => e(__e(ex))));
                        yield new Promise((r, e) => `${meta.root}/build/debug/extension.json`
                            .asFileHandle()
                            .setCache(meta.meta)
                            .write("object")
                            .then((data) => r(data))
                            .catch((ex_1) => e(__e(ex_1))));
                        yield this.copy((() => {
                            const result1 = [];
                            for (v of meta.copies) {
                                result1.push(`${meta.root}/${v}`);
                            }
                            return result1;
                        })(), `${meta.root}/build/debug`);
                        return resolve();
                    }
                    catch (e) {
                        return reject(__e(e));
                    }
                }
                catch (e_1) {
                    return reject(__e(e_1));
                }
            }));
        }
        /**
         *
         *
         * @private
         * @param {GenericObject<any>} meta
         * @returns {Promise<any>}
         * @memberof ExtensionMaker
         */
        run(meta) {
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                const path = `${meta.root}/build/debug/${meta.meta.name}.js`;
                if (OS.API.shared[path]) {
                    delete OS.API.shared[path];
                }
                try {
                    yield OS.API.requires(path);
                    let v;
                    if (this.app.extensions[meta.meta.name]) {
                        this.app.extensions[meta.meta.name].child = [];
                        for (v of meta.meta.actions) {
                            this.app.extensions[meta.meta.name].addAction(v);
                        }
                    }
                    else {
                        this.app.extensions[meta.meta.name] = new App.CMDMenu(meta.meta.text);
                        this.app.extensions[meta.meta.name].name =
                            meta.meta.name;
                        for (v of meta.meta.actions) {
                            this.app.extensions[meta.meta.name].addAction(v);
                        }
                        this.app.spotlight.addAction(this.app.extensions[meta.meta.name]);
                        this.app.extensions[meta.meta.name].onchildselect((e) => {
                            return this.app.loadAndRunExtensionAction(e.data.item.data);
                        });
                    }
                    this.app.spotlight.run(this.app);
                    return resolve();
                }
                catch (e) {
                    return reject(__e(e));
                }
            }));
        }
        /**
         *
         *
         * @private
         * @param {string[]} files
         * @param {*} zip
         * @returns {Promise<any>}
         * @memberof ExtensionMaker
         */
        installExtension(files, zip) {
            return new Promise((resolve, reject) => {
                const idx = files.indexOf("extension.json");
                if (idx < 0) {
                    reject(OS.API.throwe(__("No meta-data found")));
                }
                const metafile = files.splice(idx, 1)[0];
                // read the meta file
                return zip
                    .file(metafile)
                    .async("uint8array")
                    .then((d) => {
                    const meta = JSON.parse(new TextDecoder("utf-8").decode(d));
                    return this.installFiles(files, zip, meta)
                        .then(() => resolve())
                        .catch((e) => reject(__e(e)));
                })
                    .catch((e) => reject(__e(e)));
            });
        }
        /**
         *
         *
         * @private
         * @param {string[]} files
         * @param {*} zip
         * @param {GenericObject<any>} meta
         * @returns {Promise<any>}
         * @memberof ExtensionMaker
         */
        installFiles(files, zip, meta) {
            if (files.length === 0) {
                return this.installMeta(meta);
            }
            return new Promise((resolve, reject) => {
                const file = files.splice(0, 1)[0];
                const path = `${this.basedir()}/${file}`;
                return zip
                    .file(file)
                    .async("uint8array")
                    .then((d) => {
                    return path
                        .asFileHandle()
                        .setCache(new Blob([d], { type: "octet/stream" }))
                        .write("text/plain")
                        .then((r) => {
                        if (r.error) {
                            return reject(r.error);
                        }
                        return this.installFiles(files, zip, meta)
                            .then(() => resolve())
                            .catch((e) => reject(__e(e)));
                    })
                        .catch((e) => reject(__e(e)));
                })
                    .catch((e) => reject(__e(e)));
            });
        }
        /**
         *
         *
         * @private
         * @param {GenericObject<any>} meta
         * @returns {Promise<any>}
         * @memberof ExtensionMaker
         */
        installMeta(meta) {
            return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                const file = `${this.app.meta().path}/extensions.json`.asFileHandle();
                try {
                    const data = yield file.read("json");
                    const names = [];
                    for (let v of data) {
                        names.push(v.name);
                    }
                    const idx = names.indexOf(meta.name);
                    if (idx >= 0) {
                        data.splice(idx, 1);
                    }
                    data.push(meta);
                    try {
                        yield file.setCache(data).write("object");
                        return resolve();
                    }
                    catch (e) {
                        return reject(__e(e));
                    }
                }
                catch (e_1) {
                    return reject(__e(e_1));
                }
            }));
        }
        /**
         *
         *
         * @private
         * @param {string} path
         * @returns {Promise<any>}
         * @memberof ExtensionMaker
         */
        installZip(path) {
            return new Promise((resolve, reject) => {
                this.import(["os://scripts/jszip.min.js"])
                    .then(() => {
                    path.asFileHandle()
                        .read("binary")
                        .then((data) => {
                        JSZip.loadAsync(data)
                            .then((zip) => {
                            const pth = this.basedir();
                            const dir = [];
                            const files = [];
                            for (let name in zip.files) {
                                const file = zip.files[name];
                                if (file.dir) {
                                    dir.push(pth + "/" + name);
                                }
                                else {
                                    files.push(name);
                                }
                            }
                            if (dir.length > 0) {
                                this.mkdirAll(dir)
                                    .then(() => {
                                    this.installExtension(files, zip)
                                        .then(() => resolve())
                                        .catch((e) => reject(__e(e)));
                                })
                                    .catch((e) => reject(__e(e)));
                            }
                            else {
                                this.installExtension(files, zip)
                                    .then(() => resolve())
                                    .catch((e) => reject(__e(e)));
                            }
                        })
                            .catch((e) => reject(__e(e)));
                    })
                        .catch((e) => reject(__e(e)));
                })
                    .catch((e) => reject(__e(e)));
            });
        }
    }
    App.extensions.ExtensionMaker = ExtensionMaker;
})(OS || (OS = {}));
